@@ -13,6 +13,7 @@ import sys
 import logging
 from errno import EPERM, ENOENT, EACCES, ENOTEMPTY, ENOTDIR, EIO
 import cloudfiles
+from cloudfiles import Connection
 from chunkobject import ChunkObject
 from errors import IOSError
 import posixpath
@@ -21,6 +22,21 @@ from functools import wraps
 
 __all__ = ['CloudFilesFS']
 
+class ProxyConnection(Connection):
+    """
+    Add X-Forwarded-For header to all requests.
+    """
+    def __init__(self, *args, **kwargs):
+        self.real_ip = None
+        super(ProxyConnection, self).__init__(*args, **kwargs)
+
+    def make_request(self, method, path=[], data='', hdrs=None, parms=None):
+        if self.real_ip:
+            if not hdrs:
+                hdrs = {}
+            hdrs['X-Forwarded-For'] = self.real_ip
+        return super(ProxyConnection, self).make_request(method, path, data, hdrs, parms)
+
 def translate_cloudfiles_error(fn):
     """
     Decorator to catch cloudfiles errors and translating them into IOSError.
@@ -308,7 +324,7 @@ class CloudFilesFS(object):
         # compatibility with old python api versions
         if self.authurl:
             kwargs['authurl'] = self.authurl
-        self.connection = cloudfiles.get_connection(username, api_key, timeout=30, **kwargs)
+        self.connection = ProxyConnection(username, api_key, timeout=30, **kwargs)
 
     def close(self):
         '''Dummy function which does nothing - no need to close'''
@@ -606,3 +622,22 @@ class CloudFilesFS(object):
         e = "readlink %r - not implemented" % path
         logging.debug(e)
         raise IOSError(EPERM, 'Operation not permitted: %s' % e)
+
+    @translate_cloudfiles_error
+    def md5(self, path):
+        '''Return the object MD5 for path, raise OSError on error'''
+        path = self.abspath(path)
+        logging.debug("md5 %r" % path)
+        container, name = parse_fspath(path)
+
+        if not name:
+            raise IOSError(EACCES, "Can't return the MD5 of a container")
+
+        if self.isdir(path):
+            # this is only 100% accurate for virtual directories
+            raise IOSError(EACCES, "Can't return the MD5 of a directory")
+
+        container = self._get_container(container)
+        obj = container.get_object(name)
+        return obj.etag
+
