@@ -16,15 +16,24 @@ from server import RackspaceCloudFilesFS
 from constants import version, default_address, default_port, \
     default_config_file, default_banner, default_workers
 from monkeypatching import MyFTPHandler, MyDTPHandler
+from multiprocessing import Manager
 
-def remove_unsupported_ftp_commands():
-    """Remove ftp commands we don't / can't support"""
+def modify_supported_ftp_commands():
+    """Remove the FTP commands we don't / can't support, and add the extensions"""
     unsupported = (
         'SITE CHMOD',
     )
     for cmd in unsupported:
         if cmd in ftpserver.proto_cmds:
             del ftpserver.proto_cmds[cmd]
+    # add the MD5 command, FTP extension according to IETF Draft:
+    # http://tools.ietf.org/html/draft-twine-ftpmd5-00
+    ftpserver.proto_cmds.update({
+        'MD5': dict(perm=None,
+                    auth=True,
+                    arg=True,
+                    help='Syntax: MD5 <SP> file-name (get MD5 of file)')
+        })
 
 def start_garbage_collector(interval=10):
     """Starts the garbage collector at the interval in seconds. 0 means
@@ -87,6 +96,7 @@ class Main(object):
                                   'port': default_port,
                                   'bind-address': default_address,
                                   'workers': default_workers,
+                                  'max-cons-per-ip': '0',
                                   'auth-url': None,
                                   'service-net': 'no',
                                   'verbose': 'no',
@@ -205,6 +215,13 @@ class Main(object):
             except socket.gaierror, (_, errmsg):
                 sys.exit('Masquerade address error: %s' % errmsg)
 
+        try:
+            max_cons_per_ip = int(self.config.get('ftpcloudfs', 'max-cons-per-ip'))
+        except ValueError, errmsg:
+            sys.exit('Max connections per IP error: %s' % errmsg)
+
+        ftpserver.FTPServer.max_cons_per_ip = max_cons_per_ip
+
         ftpd = ftpserver.FTPServer((self.options.bind_address,
                                     self.options.port),
                                    MyFTPHandler)
@@ -241,6 +258,9 @@ class Main(object):
                 os.kill(pid, signal)
             except:
                 pass
+
+        if self.shm_manager:
+            self.shm_manager.shutdown()
         self.old_signal_handler(signal, frame)
 
     def main(self):
@@ -248,11 +268,12 @@ class Main(object):
         self.pid = os.getpid()
         self.parse_configuration()
         self.parse_arguments()
-        remove_unsupported_ftp_commands()
+        modify_supported_ftp_commands()
 
         ftpd = self.setup_server()
 
         if self.options.foreground:
+            MyFTPHandler.shared_ip_map = None
             self.setup_log()
             ftpd.serve_forever()
             return
@@ -261,11 +282,17 @@ class Main(object):
         daemonContext = self.setup_daemon([ftpd.socket.fileno(),])
         with daemonContext:
             self.old_signal_handler = signal.signal(signal.SIGTERM, self.signal_handler)
+
+            self.shm_manager = Manager()
+            MyFTPHandler.shared_ip_map = self.shm_manager.dict()
+            MyFTPHandler.shared_lock = self.shm_manager.Lock()
+
             for i in range(self.options.workers):
                 pid = os.fork()
                 if pid == 0:
                     self.pid = os.getpid()
                     self.pidfile.close()
+                    self.shm_manager = None
                     break
                 self._workers.append(pid)
             self.setup_log()
